@@ -853,7 +853,8 @@ int switchToArticleMode()
     for (i = FirstListedArticle + 1; i <= newsgroup->last; i++) {
       art = artStructGet(newsgroup, i, False);
       if (IS_AVAIL(art) && IS_UNREAD(art)) {
-	struct article copy = *art;
+	struct article copy;
+	copy = *art;
 	SET_LISTED(&copy);
 	artStructReplace(newsgroup, &art, &copy, i);
       }
@@ -2128,7 +2129,8 @@ static String artKillSession(
 		cur_val = *(char **)((char *)art + target_offset);
 		if (! cur_val) {
 		  /* XXX See above. */
-		    struct article copy = *art;
+		    struct article copy;
+		    copy = *art;
 		    mesgPane(XRN_SERIOUS, 0, ART_NOT_AVAIL_MSG, artNum);
 		    TextRemoveLine(SubjectText, left);
 		    removeLine(SubjectString, &left);
@@ -2274,37 +2276,6 @@ void artKillAuthorFunction(widget, event, string, count)
 		       event, string, count, True);
 }
 
-static char *getFirstReference _ARGUMENTS((char *));
-
-static char *getFirstReference(references)
-     char *references;
-{
-  static char *ref_buf = NULL;
-  static int buf_size = 0;
-  char *lbrace, *rbrace;
-
-  if (! (references && *references))
-    return NULL;
-
-  if (! (lbrace = strchr(references, '<')))
-    return NULL;
-
-  if (! (rbrace = strchr(lbrace, '>')))
-    return NULL;
-
-  rbrace++;
-
-  if (rbrace - lbrace + 1 > buf_size) {
-    buf_size = rbrace - lbrace + 1;
-    ref_buf = XtRealloc(ref_buf, buf_size);
-  }
-
-  strncpy(ref_buf, lbrace, rbrace - lbrace);
-  ref_buf[rbrace - lbrace] = '\0';
-
-  return ref_buf;
-}
-
 void artKillThreadFunction(widget, event, string, count)
      Widget widget;
      XEvent *event;
@@ -2357,7 +2328,7 @@ void artListOldHandler(widget, client_data, call_data)
     TextUnsetSelection(SubjectText);
 
     if ((int) client_data == XRNgotoArticle_ABORT) {
-      goto finished;
+      goto finishedl;
     }
     else if ((int) client_data == XRNgotoArticle_DEFAULT) {
       artNum = CurrentGroup->first;
@@ -2373,7 +2344,7 @@ void artListOldHandler(widget, client_data, call_data)
 	  artNum = atol(numberstr);
 	if (! artNum) {
 	  mesgPane(XRN_SERIOUS, 0, BAD_ART_NUM_MSG, numberstr);
-	  goto finished;
+	  goto finishedl;
 	}
 	if (*numberstr == '+')
 	  artNum = MAX(FirstListedArticle - artNum, CurrentGroup->first);
@@ -2422,7 +2393,7 @@ void artListOldHandler(widget, client_data, call_data)
       setListSorted(True);
     }
 
-finished:
+finishedl:
     if (GotoArticleBox) {
       PopDownDialog(GotoArticleBox);
       GotoArticleBox = 0;
