@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: compose.c,v 1.122 1998/05/14 01:29:15 jik Exp $";
+static char XRNrcsid[] = "$Id: compose.c,v 1.126 1999/09/24 14:00:55 jik Exp $";
 #endif
 
 /*
@@ -95,6 +95,7 @@ extern int fclose _ARGUMENTS((FILE *));
 #include "mesg_strings.h"
 
 #ifdef INN
+#include <configdata.h>
 #include <clibrary.h>
 #include <libinn.h>
 #endif
@@ -139,6 +140,7 @@ static char *followup_or_reply_title _ARGUMENTS((struct header *, int, int));
 static void compose_buttons _ARGUMENTS((Widget, int));
 static int check_quoted_text _ARGUMENTS((char *));
 static char *insert_courtesy_tag _ARGUMENTS((char *));
+static Boolean IsValidAddressField _ARGUMENTS((CONST char *));
 
 #ifdef GENERATE_EXTRA_FIELDS
 static char *gen_id _ARGUMENTS((void));
@@ -269,6 +271,7 @@ char *getUser()
   
   host:
   * Contents of the HIDDENHOST environment variable, if it's set, or
+  * Contents of the hiddenHost X resource, if it's set, or
   * INN "fromhost" configuration value, if using INN and it's set, or
   * Contents of the file HIDDEN_FILE, if it's defined and non-empty, or
   * The string RETURN_HOST, if it's defined, or
@@ -282,6 +285,7 @@ char *getUser()
   * Contents of real_host
   
   host's domain: (only if host doesn't already have a period in it)
+  * Contents of the "domainName" X resource, if it's set, or
   * Contents of the DOMAIN environment variable, if it's set, or
   * INN "domain" configuration value, if using INN and it's set, or
   * Contents of the file DOMAIN_FILE, if it's defined and non-empty, or
@@ -1399,14 +1403,41 @@ void compSendFunction(widget, event, string, count)
 	    buffer2_size = 0;
 	    buildRealFrom(&Header, &buffer2, &buffer2_size, &buffer2_total_size);
 	    if (strcmp(buffer, buffer2)) {
+	      char *addr, *addr_buf, *end, *nl;
+
+	      addr = strchr(buffer, ':') + 1;
+	      addr_buf = addr = XtNewString(addr);
+
+	      /* Get rid of newlines */
+	      while ((nl = strchr(addr, '\n')))
+		*nl = ' ';
+
+	      /* Strip whitespace in front */
+	      while (isspace((unsigned char) *addr))
+		addr++;
+
+	      /* Strip whitespace at the end */
+	      for (end = strchr(addr, '\0') - 1;
+		   end > addr && isspace((unsigned char) *end);
+		   end--)
+		*end = '\0';
+
+	      /* Now make sure its basic syntax is correct */
+	      if (! IsValidAddressField(addr)) {
+		  XBell(XtDisplay(TopLevel), 0);
+		  mesgPane(XRN_SERIOUS, 0, BAD_FROM_MSG, addr);
+		  XtFree(addr_buf);
+		  if (app_resources.editorCommand)
+		    Call_Editor(True, False);
+		  TextEnableRedisplay(ComposeText);
+		  CLEANUP(); return;
+	      }
+
 #ifdef SENDMAIL_VERIFY
 	      if (app_resources.verifyFrom) {
-		char *verify_command, *addr, *addr_buf, *ptr, *ptr2;
+		char *verify_command, *ptr, *ptr2;
 		int cmd_length;
 
-		addr = strchr(buffer, ':') + 1;
-		addr_buf = addr = XtNewString(addr);
-		
 		while (isspace((unsigned char)*addr))
 		  addr++;
 		if ((ptr = strchr(addr, '\n'))) {
@@ -1423,21 +1454,16 @@ void compSendFunction(widget, event, string, count)
 		verify_command = XtMalloc(cmd_length);
 		(void) strcpy(verify_command, SENDMAIL_VERIFY);
 		ptr = &verify_command[sizeof(SENDMAIL_VERIFY)-1];
-		*ptr++ = ' ';
-		*ptr++ = '\'';
+		*ptr++ = ' '; *ptr++ = '\'';
 		for (ptr2 = addr; *ptr2; ptr2++) {
 		  if (*ptr2 == '\'' || *ptr2 == '\\') {
-		    *ptr++ = '\'';
-		    *ptr++ = '\\';
-		    *ptr++ = *ptr2;
-		    *ptr++ = '\'';
+		    *ptr++ = '\''; *ptr++ = '\\'; *ptr++ = *ptr2; *ptr++ = '\'';
 		  }
 		  else {
 		    *ptr++ = *ptr2;
 		  }
 		}
-		*ptr++ = '\'';
-		*ptr++ = '\0';
+		*ptr++ = '\''; *ptr++ = '\0';
 
 		if (system(verify_command)) {
 		  XBell(XtDisplay(TopLevel), 0);
@@ -1446,18 +1472,16 @@ void compSendFunction(widget, event, string, count)
 		  if (app_resources.editorCommand)
 		    Call_Editor(True, False);
 		  TextEnableRedisplay(ComposeText);
-		  CLEANUP();
-		  return;
+		  CLEANUP(); return;
 		}
-
-		XtFree(addr_buf);
 	      }
 #endif /* SENDMAIL_VERIFY */
 
-		*buffer = '\0';
-		buffer_size = 0;
-		buildSender(&Header, &buffer, &buffer_size, &buffer2_size);
-		addField(buffer);
+	      XtFree(addr_buf);
+	      *buffer = '\0';
+	      buffer_size = 0;
+	      buildSender(&Header, &buffer, &buffer_size, &buffer2_size);
+	      addField(buffer);
 	    }
 	}
 #endif /* ! INEWS */	
@@ -3581,3 +3605,186 @@ static char *insert_courtesy_tag(message)
   XtFree(message);
   return(new_message);
 }
+
+/*
+  Converts address fields as follows:
+
+  From			To
+  ----			--
+  address		address
+  stuff1 <address>	address
+  address (stuff2)	address
+
+  Makes sure that "address" contains '@', contains only valid
+  characters after the '@', and is quoted if it contains characters
+  before the '@' that need quoting.  Makes sure that "stuff1" either
+  is quoted or doesn't contain a period or comma.  Makes sure that
+  "stuff1" and "stuff2" don't contain parentheses or angle brackets.
+  Returns true if all this is OK or false otherwise.
+*/
+
+static Boolean check_address _ARGUMENTS((CONST char *));
+static Boolean check_stuff _ARGUMENTS((CONST char *, Boolean));
+
+static Boolean IsValidAddressField(value_in)
+     CONST char *value_in;
+{
+  char *value, *value_buf, *address, *stuff, *ptr, *end;
+  Boolean retval = True;
+
+  value_buf = value = XtNewString(value_in);
+  address = stuff = NULL;
+
+  while ((ptr = strchr(value, '\n')))
+    *ptr = ' ';
+  while (isspace((unsigned char) *value))
+    value++;
+  for (end = strchr(value, '\0'), ptr = end - 1;
+       ptr > value && isspace((unsigned char) *ptr);
+       ptr--, end--)
+    *ptr = '\0';
+
+  /* Is it stuff1 <address>? */
+  ptr = end - 1;
+  if (ptr > value && *ptr == '>') {
+    *ptr-- = '\0';
+    for (ptr = value; *ptr && *ptr != '<'; ptr++)
+      /* empty */;
+    if (*ptr != '<') {
+      retval = False;
+      goto finished;
+    }
+    address = ptr + 1;
+    stuff = value;
+    *ptr-- = '\0';
+    while (ptr > stuff && isspace((unsigned char) *ptr))
+      *ptr-- = '\0';
+    if (! (retval = check_address(address)))
+      goto finished;
+    if (! (retval = check_stuff(stuff, True)))
+      goto finished;
+  }
+  /* Is it address (stuff2)? */
+  else if (ptr > value && *ptr == ')') {
+    *ptr-- = '\0';
+    while (ptr > value && *ptr != '(')
+      ptr--;
+    if (*ptr != '(') {
+      retval = False;
+      goto finished;
+    }
+    address = value;
+    stuff = ptr + 1;
+    *ptr-- = '\0';
+    while (ptr > address && isspace((unsigned char) *ptr))
+      *ptr-- = '\0';
+    if (! (retval = check_address(address)))
+      goto finished;
+    if (! (retval = check_stuff(stuff, False)))
+      goto finished;
+  }
+  /* Otherwise, it's just an address */
+  else {
+    retval = check_address(value);
+  }
+
+ finished:
+  XtFree(value_buf);
+  return retval;
+}
+
+/*
+  Make sure the address is in the form local-part@domain.  If
+  local-part contains characters that need quoting, make sure that it
+  is surrounded by quotation marks.  Returns True if the address is OK
+  or False otherwise.
+*/
+
+static Boolean check_address(address)
+     CONST char *address;
+{
+  static char special_chars[] = "()<>@,;:\\\"[] ";
+  Boolean in_quotes, in_quoted_pair, in_local_part, found_dot;
+  CONST char *ptr;
+
+  in_quotes = in_quoted_pair = found_dot = False;
+  in_local_part = True;
+
+  for (ptr = address; *ptr; ptr++) {
+    if (in_local_part) {
+      if (in_quoted_pair) {
+	in_quoted_pair = False;
+	continue;
+      }
+      if (*ptr == '\\') {
+	in_quoted_pair = True;
+	continue;
+      }
+      if (in_quotes) {
+	if (*ptr == '"')
+	  in_quotes = False;
+	continue;
+      }
+      if (*ptr == '"') {
+	in_quotes = True;
+	continue;
+      }
+      if (*ptr == '@') {
+	if (ptr == address) /* No local part! */
+	  return False;
+	in_local_part = False;
+	continue;
+      }
+    }
+    if (strchr(special_chars, *ptr))
+      return False;
+    if (in_local_part)
+      continue;
+    if (*ptr == '.')
+      found_dot++;
+  }
+
+  if (found_dot)
+    return True;
+
+  return False;
+}
+
+/*
+  Make sure that "stuff" doesn't contain parentheses, angle
+  brackets, or internal quotation marks.  Furthermore, if
+  "needs_quoting" is true, make sure that it is enclosed in quotation
+  marks if it contains periods or commas.
+*/
+
+static Boolean check_stuff(
+			   _ANSIDECL(CONST char *,	stuff),
+			   _ANSIDECL(Boolean,		needs_quoting)
+			   )
+     _KNRDECL(CONST char *,	stuff)
+     _KNRDECL(Boolean,		needs_quoting)
+{
+  static char specials[] = "()<>\"";
+  static char quoted_specials[] = ".,";
+
+  CONST char *ptr, *end;
+  Boolean is_quoted = False;
+
+  end = strchr(stuff, '\0');
+
+  if (*stuff == '"') {
+    if (*--end != '"')
+      return False;
+    stuff++;
+    is_quoted = True;
+  }
+
+  for (ptr = stuff; ptr < end; ptr++) {
+    if (strchr(specials, *ptr))
+      return False;
+    if (needs_quoting && !is_quoted && strchr(quoted_specials, *ptr))
+      return False;
+  }
+
+  return True;
+}
