@@ -26,15 +26,15 @@
 #define DEF_CHECK_FLAGS		(KILL_SUBJECT|KILL_AUTHOR)
 #define ALL_CHECK_FLAGS		(KILL_SUBJECT|KILL_AUTHOR|\
 				 KILL_NEWSGROUPS|KILL_DATE|\
-				 KILL_ID|KILL_REFERENCES)
+				 KILL_ID|KILL_REFERENCES|KILL_APPROVED)
 
 static void free_entry_contents _ARGUMENTS((kill_entry *));
-static char field_to_check_flag _ARGUMENTS((char *, int));
+static kill_check_flag_t field_to_check_flag _ARGUMENTS((char *, int));
 static Boolean entry_expired _ARGUMENTS((kill_entry *, time_t));
 static void write_kf _ARGUMENTS((kill_file *));
 static kill_file *read_kf _ARGUMENTS((char *file_name,
 				      char *reference,
-				      unsigned char *fetch_flags));
+				      fetch_flag_t *fetch_flags));
 
 
 /*
@@ -68,7 +68,7 @@ static Boolean isQuoted(character, start)
 
 static void parse_kill_entry _ARGUMENTS((char *, char *, kill_file *,
 					 kill_entry *,
-					 unsigned char *, int));
+					 fetch_flag_t *, int));
 
 /*
   Parse a KILL-file entry.  If there's a parse error, this routine
@@ -107,7 +107,7 @@ static void parse_kill_entry(file_name, in_str, file, entry,
      char *in_str;
      kill_file *file;
      kill_entry *entry;
-     unsigned char *fetch_flags;
+     fetch_flag_t *fetch_flags;
      int mesg_name;
 {
   kill_entry my_entry;
@@ -316,6 +316,12 @@ static void parse_kill_entry(file_name, in_str, file, entry,
   case 's':
     my_entry.entry.action_flags = KILL_SAVE;
     break;
+  case 't':
+    my_entry.entry.action_flags = KILL_SUBTHREAD;
+    break;
+  case 'T':
+    my_entry.entry.action_flags = KILL_THREAD;
+    break;
   default:
     mesgPane(XRN_SERIOUS, mesg_name, MALFORMED_KILL_ENTRY_MSG, str,
 	     file_name, ERROR_REGEX_UNKNOWN_COMMAND_MSG);
@@ -360,6 +366,13 @@ static void parse_kill_entry(file_name, in_str, file, entry,
     *fetch_flags |= FETCH_REFS;
   if (my_entry.entry.check_flags & KILL_XREF)
     *fetch_flags |= FETCH_XREF;
+  if (my_entry.entry.check_flags & KILL_APPROVED)
+    *fetch_flags |= FETCH_APPROVED;
+
+  if (my_entry.entry.action_flags == KILL_SUBTHREAD)
+    *fetch_flags |= FETCH_IDS;
+  if (my_entry.entry.action_flags == KILL_THREAD)
+    *fetch_flags |= FETCH_IDS & FETCH_REFS;
 
   my_entry.type = KILL_ENTRY;
 
@@ -375,7 +388,7 @@ done:
 
 struct kftab_entry {
   char ref_count;
-  unsigned char fetch_flags;
+  fetch_flag_t fetch_flags;
   kill_file *kill_file;
 };
 
@@ -470,6 +483,12 @@ static void unparse_kill_entry(entry, buffer)
     case KILL_SAVE:
       *ptr++ = 's';
       break;
+    case KILL_SUBTHREAD:
+      *ptr++ = 't';
+      break;
+    case KILL_THREAD:
+      *ptr++ = 'T';
+      break;
     default:
       assert(0);
     }
@@ -483,7 +502,7 @@ static void unparse_kill_entry(entry, buffer)
 
 
 static kill_file *GlobalKillFile = 0;
-static unsigned char GlobalFetchFlags = 0;
+static fetch_flag_t GlobalFetchFlags = 0;
 
 
 void read_global_kill_file(newsgroup)
@@ -513,7 +532,7 @@ void read_local_kill_file(newsgroup)
 static kill_file *read_kf(file_name, reference, fetch_flags)
      char *file_name;
      char *reference;
-     unsigned char *fetch_flags;
+     fetch_flag_t *fetch_flags;
 {
   int entries_size;
   FILE *fp;
@@ -614,24 +633,26 @@ static kill_file *read_kf(file_name, reference, fetch_flags)
   return kf;
 }
 
-static kill_entry *kf_iter _ARGUMENTS((kill_file *, kill_entry *,
+static kill_entry *kf_iter _ARGUMENTS((kill_file *, kill_file_iter_handle *,
 				       Boolean));
 
 static kill_entry *kf_iter(
-			   _ANSIDECL(kill_file *,	kf),
-			   _ANSIDECL(kill_entry *,	last_entry),
-			   _ANSIDECL(Boolean,		expand_includes)
+			   _ANSIDECL(kill_file *,		kf),
+			   _ANSIDECL(kill_file_iter_handle *,	handle_p),
+			   _ANSIDECL(Boolean,			expand_includes)
 			   )
-     _KNRDECL(kill_file *,	kf)
-     _KNRDECL(kill_entry *,	last_entry)
-     _KNRDECL(Boolean,		expand_includes)
+     _KNRDECL(kill_file *,		kf)
+     _KNRDECL(kill_file_iter_handle *,	handle_p)
+     _KNRDECL(Boolean,			expand_includes)
 {
   kill_entry *sub_entry = 0;
+  int handle = (int) *handle_p;
+  kill_entry *this_entry;
 
   assert(kf);
 
-  if (! last_entry) {
-    last_entry = kf->entries;
+  if (! handle_p) {
+    handle = 1;
     if (expand_includes)
       kf->flags |= KF_SEEN;
   }
@@ -639,47 +660,54 @@ static kill_entry *kf_iter(
     assert(expand_includes);
     assert(kf->cur_entry);
 
-    sub_entry = kf_iter(kf->cur_sub_kf, last_entry, expand_includes);
+    sub_entry = kf_iter(kf->cur_sub_kf, handle_p, expand_includes);
 
     if (sub_entry)
       return sub_entry;
 
     kf->cur_sub_kf = 0;
-    last_entry = kf->cur_entry;
-    last_entry++;
+    handle = kf->cur_entry;
+    handle++;
   } else {
-    last_entry++;
+    handle++;
   }
 
-  if (last_entry - kf->entries == kf->count)
+  if (handle > kf->count)
     return 0;
 
+  this_entry = &kf->entries[handle-1];
+
 #if !defined(POSIX_REGEX) && !defined(SYSV_REGEX)
-  if (last_entry->type == KILL_ENTRY)
-    (void) re_comp(last_entry->entry.reStruct);
+  if (this_entry->type == KILL_ENTRY)
+    (void) re_comp(this_entry->entry.reStruct);
 #endif
 
-  if ((last_entry->type == KILL_INCLUDE) && expand_includes) {
-    if (! (last_entry->include.kf->flags & KF_SEEN)) {
-      sub_entry = kf_iter(last_entry->include.kf, 0, expand_includes);
+  if ((this_entry->type == KILL_INCLUDE) && expand_includes) {
+    if (! (this_entry->include.kf->flags & KF_SEEN)) {
+      kill_file_iter_handle sub_handle = 0;
+
+      sub_entry = kf_iter(this_entry->include.kf, &sub_handle, expand_includes);
 
       if (sub_entry) {
-	kf->cur_entry = last_entry;
-	kf->cur_sub_kf = last_entry->include.kf;
+	kf->cur_entry = handle;
+	kf->cur_sub_kf = this_entry->include.kf;
+	*handle_p = sub_handle;
 	return sub_entry;
       }
     }
 
-    return kf_iter(kf, last_entry, expand_includes);
+    *handle_p = (kill_file_iter_handle) handle;
+    return kf_iter(kf, handle_p, expand_includes);
   }
 
-  return last_entry;
+  *handle_p = (kill_file_iter_handle) handle;
+  return this_entry;
 }
 
-kill_entry *kill_file_iter(newsgroup, mode, last_entry)
+kill_entry *kill_file_iter(newsgroup, mode, handle)
      struct newsgroup *newsgroup;
      int mode;
-     kill_entry *last_entry;
+     kill_file_iter_handle *handle;
 {
   kill_file *kf;
 
@@ -690,7 +718,7 @@ kill_entry *kill_file_iter(newsgroup, mode, last_entry)
 
   clear_seen();
 
-  return kf_iter(kf, last_entry, True);
+  return kf_iter(kf, handle, True);
 }
 
 #define CHECK_WRITE(cmd) if (! (cmd)) { \
@@ -707,7 +735,8 @@ static void write_kf(kf)
   char *temp_file = 0;
   FILE *fp;
   char buf[MAX_KILL_ENTRY_LENGTH];
-  kill_entry *entry = 0;
+  kill_entry *entry;
+  kill_file_iter_handle handle = 0;
   time_t now = time(0);
   int mesg_name = newMesgPaneName();
 
@@ -745,7 +774,7 @@ static void write_kf(kf)
       CHECK_WRITE(fprintf(fp, "THRU %ld\n", kf->thru) != EOF);
     }
 
-    while ((entry = kf_iter(kf, entry, False))) {
+    while ((entry = kf_iter(kf, &handle, False))) {
       if (entry_expired(entry, now))
 	continue;
       unparse_kill_entry(entry, buf);
@@ -769,7 +798,8 @@ done:
     XtFree(temp_file);
   }
 
-  while ((entry = kf_iter(kf, entry, False)))
+  handle = 0;
+  while ((entry = kf_iter(kf, &handle, False)))
     free_entry_contents(entry);
   XtFree((char *)kf->entries);
   XtFree((char *)kf);
@@ -844,7 +874,7 @@ void add_kill_entry(newsgroup, mode, field, regexp)
   char *file;
   kill_entry my_entry;
   int mesg_name = newMesgPaneName();
-  unsigned char *fetch_ptr;
+  fetch_flag_t *fetch_ptr;
   struct stat statbuf;
 
   if (mode == KILL_LOCAL) {
@@ -924,7 +954,7 @@ void add_kill_entry(newsgroup, mode, field, regexp)
     return(flag); \
   }
 
-static char field_to_check_flag(field_name, field_length)
+static kill_check_flag_t field_to_check_flag(field_name, field_length)
      char *field_name;
      int field_length;
 {
@@ -935,6 +965,7 @@ static char field_to_check_flag(field_name, field_length)
   CHECK_FIELD("Message-ID", KILL_ID);
   CHECK_FIELD("References", KILL_REFERENCES);
   CHECK_FIELD("Xref", KILL_XREF);
+  CHECK_FIELD("Approved", KILL_APPROVED);
 
   return 0;
 }
