@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: mesg_strings.c,v 1.88 1998/03/23 13:21:31 jik Exp $";
+static char XRNrcsid[] = "$Id: mesg_strings.c,v 1.91 1998/12/30 14:25:53 jik Exp $";
 #endif
 
 /*
@@ -713,12 +713,16 @@ char *message_strings[] = {
     "marked unread - %s",  /* subject */
 /* < KILL_SAVED > */
     "saved - %s",         /* subject */
+/* < KILL_THREAD > */
+    "killed thread - %s", /* subject */
 /* < COUNT_KILLED > */
     "killed %d article%s in %s", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_UNREAD > */
     "marked %d article%s unread in %s", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_SAVED > */
     "saved %d article%s in %s", /* count, "" or NOT_ONE_STRING , newsgroup */
+/* < COUNT_THREAD > */
+    "killed %d thread%s in %s", /* count, "" or NOT_ONE_STRING, newsgroup */
 /* < ERROR_CORNERED > */
     "XRN error in `cornered': expecting nglist to be valid\n",
 /* < ERROR_OUT_OF_MEM > */
@@ -1228,12 +1232,16 @@ char *message_strings[] = {
     "marqu non lu - %s",  /* subject */
 /* < KILL_SAVED > */
     "sauv - %s",         /* subject */
+/* < KILL_THREAD > */
+    "killed thread - %s", /* subject */
 /* < COUNT_KILLED > */
     "%d article%s tus dans %s", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_UNREAD > */
     "%d article%s marqus non lus dans %s", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_SAVED > */
     "%d article%s sauvs dans %s", /* count, "" or NOT_ONE_STRING , newsgroup */
+/* < COUNT_THREAD > */
+    "killed %d thread%s in %s", /* count, "" or NOT_ONE_STRING, newsgroup */
 /* < ERROR_CORNERED > */
     "Erreur XRN dans `cornered' : nglist aurait d tre valide\n",
 /* < ERROR_OUT_OF_MEM > */
@@ -1762,12 +1770,16 @@ char *message_strings[] = {
     "als ungelesen markiert - %s",  /* subject */
 /* < KILL_SAVED > */
     "gesichert - %s",      /* subject */
+/* < KILL_THREAD > */
+    "Serie ausgeblendet - %s", /* subject */
 /* < COUNT_KILLED > */
     "%d Artikel%s in %s ausgeblendet", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_UNREAD > */
     "%d Artikel%s in %s als ungelesen markiert", /* count, "" or NOT_ONE_STRING , newsgroup */
 /* < COUNT_SAVED > */
     "%d Artikel%s in %s abgespeichert", /* count, "" or NOT_ONE_STRING , newsgroup */
+/* < COUNT_THREAD > */
+    "killed %d thread%s in %s", /* count, "" or NOT_ONE_STRING, newsgroup */
 /* < ERROR_CORNERED > */
     "XRN Fehler in `cornered': erwarte g\374ltige nglist\n",
 /* < ERROR_OUT_OF_MEM > */
