@@ -1,6 +1,6 @@
 
 #if !defined(lint) && !defined(SABER) && !defined(GCC_WALL)
-static char XRNrcsid[] = "$Id: server.c,v 1.150 1998/05/12 17:00:24 jik Exp $";
+static char XRNrcsid[] = "$Id: server.c,v 1.153 1999/01/03 16:02:26 jik Exp $";
 #endif
 
 /*
@@ -905,8 +905,9 @@ char *newgroupsDate()
 	curtime = gmtime(&clock);
 	assert(curtime);
 	(void) sprintf(date_buf, "%02d%02d%02d %02d%02d%02d GMT",
-		       curtime->tm_year, curtime->tm_mon + 1, curtime->tm_mday,
-		       curtime->tm_hour, curtime->tm_min, curtime->tm_sec);
+		       curtime->tm_year % 100, curtime->tm_mon + 1,
+		       curtime->tm_mday, curtime->tm_hour, curtime->tm_min,
+		       curtime->tm_sec);
     }
 
     return date_buf;
@@ -1416,7 +1417,8 @@ void start_server()
 	PostingAllowed = False;
       else if (response >= 0)
 	response = handle_server_response(response, server);
-      if (response >= 0)
+      if ((response >= 0) &&
+	  !(app_resources.authenticateOnConnect && authenticate()))
 	break;
       stop_server();
       (void) sprintf(buf, FAILED_CONNECT_MSG, server);
@@ -1708,6 +1710,28 @@ Boolean getxreflist(
 		   "xref", offset, 0, 0, False);
 }
 
+Boolean getapprovedlist(
+		    _ANSIDECL(struct newsgroup *,	newsgroup),
+		    _ANSIDECL(art_num,			artfirst),
+		    _ANSIDECL(art_num,			artlast),
+		    _ANSIDECL(Boolean,			unreadonly),
+		    _ANSIDECL(int *,			max)
+		    )
+     _KNRDECL(struct newsgroup *,	newsgroup)
+     _KNRDECL(art_num,			artfirst)
+     _KNRDECL(art_num,			artlast)
+     _KNRDECL(Boolean,			unreadonly)
+     _KNRDECL(int *,			max)
+{
+    struct article foo;
+    unsigned offset;
+
+    offset = (char *)&foo.approved - (char *)&foo;
+
+    return getlist(newsgroup, artfirst, artlast, unreadonly, max,
+		   "approved", offset, 0, 0, False);
+}
+
 Boolean getdatelist(
 		    _ANSIDECL(struct newsgroup *,	newsgroup),
 		    _ANSIDECL(art_num,			artfirst),
